# frozen_string_literal: true

RSpec.describe Deeds::Grammar do

  describe '#time' do
    context 'midnight' do
      it 'is 0 minutes old' do
        input = '0:00'
        node = described_class.parse input, root: :time
        expect(node.minutes).to eq 0
      end
    end

    context 'almost tomorrow' do
      it 'is almost as old as a day has minutes' do
        input = '23:59'
        node = described_class.parse input, root: :time
        expect(node.minutes).to eq 1_439
      end
    end

    context 'leading zero time' do
      it 'knows how many minutes old it is' do
        input = '02:02'
        node = described_class.parse input, root: :time
        expect(node.minutes).to eq 122
      end
    end

    context 'whitespace' do
      it 'ignores the whitespace' do
        input = ' 02   :  5  '
        node = described_class.parse input, root: :time
        expect(node.minutes).to eq 125
      end
    end

    context 'invalid time' do
      it 'raises an exception' do
        input = '25:00'
        node = described_class.parse input, root: :time
        expect { node.minutes }.to raise_error Citrus::ParseError
      end
    end
  end

  describe '#interval' do
    context 'as long as the whole day' do
      it 'knows how many minutes that is' do
        input = '0:00-23:59'
        node = described_class.parse input, root: :interval
        expect(node.minutes).to eq 1_439
      end
    end

    context 'just a few minutes' do
      it 'knows the minutes' do
        input = '7:45-8:2'
        node = described_class.parse input, root: :interval
        expect(node.minutes).to eq 17
      end
    end

    context 'whitespace' do
      it 'ignores the whitespace' do
        input = '  7 :45 - 8: 2 '
        node = described_class.parse input, root: :interval
        expect(node.minutes).to eq 17
      end
    end

    context 'invalid beginning time' do
      it 'raises an exception' do
        input = '7:60-8:2'
        node = described_class.parse input, root: :interval
        expect { node.minutes }.to raise_error Citrus::ParseError
      end
    end

    context 'invalid ending time' do
      it 'raises an exception' do
        input = '7:04-24:01'
        node = described_class.parse input, root: :interval
        expect { node.minutes }.to raise_error Citrus::ParseError
      end
    end

    context 'both times invalid' do
      it 'raises an exception' do
        input = '24:00-25:00'
        node = described_class.parse input, root: :interval
        expect { node.minutes }.to raise_error Citrus::ParseError
      end
    end
  end

  describe '#date' do
    context 'valid date' do
      it 'is a Date instance' do
        input = '2016-05-24'
        node = described_class.parse input, root: :date
        expect(node.value).to eq Date.new(2016, 5, 24)
      end
    end

    context 'short valid date' do
      it 'is a Date instance' do
        input = '800-4-1'
        node = described_class.parse input, root: :date
        expect(node.value).to eq Date.new(800, 4, 1)
      end
    end

    context 'invalid date' do
      it 'raises an exception' do
        input = '2016-02-31'
        node = described_class.parse input, root: :date
        expect { node.value }.to raise_error Citrus::ParseError
      end
    end
  end

  describe '#number' do
    context 'simple number' do
      it 'knows the value' do
        input = '18'
        node = described_class.parse input, root: :number
        expect(node.value).to eq 18
      end
    end

    context 'negative number' do
      it 'knows the value' do
        input = '-20'
        node = described_class.parse input, root: :number
        expect(node.value).to eq(-20)
      end
    end

    context 'explicitly positive number' do
      it 'knows the value' do
        input = '+22'
        node = described_class.parse input, root: :number
        expect(node.value).to eq 22
      end
    end

    context 'explicitly positive number with whitespace' do
      it 'knows the value' do
        input = '    + 23    '
        node = described_class.parse input, root: :number
        expect(node.value).to eq 23
      end
    end
  end

  describe '#minute' do
    context 'one minute' do
      it 'knows the period' do
        input = '42m'
        node = described_class.parse input, root: :minute
        expect(node.minutes).to eq 42
      end
    end

    context 'one minute with trailing space' do
      it 'knows the sum' do
        input = '4m    '
        node = described_class.parse input, root: :minute
        expect(node.minutes).to eq 4
      end
    end

    context 'one minute with preceeding space' do
      it 'knows the sum' do
        input = '    5m'
        node = described_class.parse input, root: :minute
        expect(node.minutes).to eq 5
      end
    end

    context 'one minute with spaces around' do
      it 'knows the sum' do
        input = '    6m   '
        node = described_class.parse input, root: :minute
        expect(node.minutes).to eq 6
      end
    end

    context 'one plussed minute' do
      it 'knows the sum' do
        input = '+8m'
        node = described_class.parse input, root: :minute
        expect(node.minutes).to eq 8
      end
    end

    context 'one plussed minute with space between plus and number' do
      it 'knows the sum' do
        input = '+  9m'
        node = described_class.parse input, root: :minute
        expect(node.minutes).to eq 9
      end
    end
  end

  describe '#hour' do
    context 'one hour' do
      it 'knows the period' do
        input = '42h'
        node = described_class.parse input, root: :hour
        expect(node.minutes).to eq 2_520
      end
    end

    context 'one hour with trailing space' do
      it 'knows the sum' do
        input = '4h    '
        node = described_class.parse input, root: :hour
        expect(node.minutes).to eq 240
      end
    end

    context 'one hour with preceeding space' do
      it 'knows the sum' do
        input = '    5h'
        node = described_class.parse input, root: :hour
        expect(node.minutes).to eq 300
      end
    end

    context 'one hour with spaces around' do
      it 'knows the sum' do
        input = '    6h   '
        node = described_class.parse input, root: :hour
        expect(node.minutes).to eq 360
      end
    end
  end

  describe '#durations' do
    context 'two positive periods' do
      it 'knows the sum' do
        input = '9m 1m'
        node = described_class.parse input, root: :durations
        expect(node.minutes).to eq 10
      end
    end

    context 'multiple positive periods' do
      it 'knows the sum' do
        input = '2m 5m 1m 3m 3m'
        node = described_class.parse input, root: :durations
        expect(node.minutes).to eq 14
      end
    end

    context 'two negative periods' do
      it 'knows the sum' do
        input = '-8m -3m'
        node = described_class.parse input, root: :durations
        expect(node.minutes).to eq(-11)
      end
    end

    context 'mixed negative and positive periods' do
      it 'knows the sum' do
        input = '4m -5m +10m'
        node = described_class.parse input, root: :durations
        expect(node.minutes).to eq 9
      end
    end

    context 'one positive period' do
      it 'knows the period' do
        input = '2m'
        node = described_class.parse input, root: :durations
        expect(node.minutes).to eq 2
      end
    end
  end

  describe '.parse' do
    context 'a date and a few periods' do
      it 'knows the date and the minutes' do
        input = '2011-01-18 1h   30m 15m  -2m   5m 10m '
        node = described_class.parse input
        expect(node.date).to eq Date.new(2011, 1, 18)
        expect(node.minutes).to eq 118
      end
    end

    context 'a date and one period' do
      it 'knows the date and the minutes' do
        input = '2011-01-19    2m'
        node = described_class.parse input
        expect(node.date).to eq Date.new(2011, 1, 19)
        expect(node.minutes).to eq 2
      end
    end

    context 'just a date' do
      it 'identifies the date' do
        input = '2016-01-18'
        node = described_class.parse input
        expect(node.date).to eq Date.new(2016, 1, 18)
        expect(node.minutes).to eq 0
      end
    end

    context 'just a date and an interval' do
      it 'identifies the date and the interval' do
        input = '2016-07-18 7:00-7:09'
        node = described_class.parse input
        expect(node.date).to eq Date.new(2016, 7, 18)
        expect(node.minutes).to eq 9
      end
    end

    context 'a date an interval and durations' do
      it 'knows the date and the total minutes' do
        input = '2018-07-18 14:00-14:30 -10m 1h'
        node = described_class.parse input
        expect(node.date).to eq Date.new(2018, 7, 18)
        expect(node.minutes).to eq 80
        expect(node.activity).to be_nil
      end
    end

    context 'a date an interval and durations and an activity' do
      it 'knows the date and the total minutes' do
        input = '2001-07-1 14:00-14:30 +5m planning all 24h of the day'
        node = described_class.parse input
        expect(node.date).to eq Date.new(2001, 7, 1)
        expect(node.minutes).to eq 35
        expect(node.activity).to eq 'planning all 24h of the day'
      end
    end
  end

end
