# frozen_string_literal: true

RSpec.describe Deeds::Journal do

  describe '#invalid_entries' do
    context 'real journal' do
      it 'knows which rows are invalid' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt')
        expect(journal.invalid_entries.count).to eq 2

        expect(journal.invalid_entries[0].input).to eq '2015-10-32 105m'
        expect(journal.invalid_entries[0].line).to eq 9
        expect(journal.invalid_entries[0].error.message).to include 'valid date'

        expect(journal.invalid_entries[1].input).to eq '2016-08-23 18:30-29:15'
        expect(journal.invalid_entries[1].line).to eq 361
        expect(journal.invalid_entries[1].error.message).to include 'valid time'
      end
    end
  end

  describe '#minutes' do
    context 'real journal' do
      it 'knows the total sum' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt')
        expect(journal.minutes).to be_instance_of BigDecimal
        expect(journal.minutes).to eq 36_043
      end
    end
  end

  describe '#hours' do
    context 'real journal' do
      it 'knows the total sum' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt')
        expect(journal.hours).to be_instance_of BigDecimal
        expect(journal.hours).to be_within(0.01).of(600.71)
      end
    end
  end

  describe '#full_time_days' do
    context 'default working hours per day' do
      it 'knows the number of days' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt')
        expect(journal.full_time_days).to be_instance_of BigDecimal
        expect(journal.full_time_days).to be_within(0.01).of(75.08)
      end
    end

    context 'custom working hours per day' do
      it 'knows the number of days' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt'), hours_per_work_day: 9
        expect(journal.full_time_days).to be_instance_of BigDecimal
        expect(journal.full_time_days).to be_within(0.01).of(66.74)
      end
    end
  end

  describe '#full_time_weeks' do
    context 'no entries' do
      it 'is zero' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/nothing.txt')
        expect(journal.full_time_weeks).to be_zero
      end
    end

    context 'six days' do
      it 'is zero' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/six_days.txt')
        expect(journal.full_time_weeks).to be_instance_of BigDecimal
        expect(journal.full_time_weeks).to be_within(0.01).of(0.01)
      end
    end

    context 'default work days per week' do
      it 'knows the number of days' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt')
        expect(journal.full_time_weeks).to be_instance_of BigDecimal
        expect(journal.full_time_weeks).to be_within(0.01).of(15.01)
      end
    end

    context 'custom work days per week' do
      it 'knows the number of days' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt'), work_days_per_week: 4
        expect(journal.full_time_weeks).to be_instance_of BigDecimal
        expect(journal.full_time_weeks).to be_within(0.01).of(18.77)
      end
    end
  end

  describe '#full_time_months' do
    context 'real journal' do
      it 'knows the number of months' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/journal.txt')
        expect(journal.full_time_months).to be_instance_of BigDecimal
        expect(journal.full_time_months).to be_within(0.01).of(3.75)
      end
    end
  end

  describe '#average_minutes_per_day' do
    context 'no entries' do
      it 'is zero' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/nothing.txt')
        expect(journal.average_minutes_per_day).to be_zero
      end
    end

    context 'over two weeks' do
      it 'knows the average minutes per 7-day-week' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/two_weeks.txt')
        expect(journal.average_minutes_per_day).to be_instance_of BigDecimal
        expect(journal.average_minutes_per_day).to be_within(0.01).of(8.78)
      end
    end
  end

  describe '#average_hours_per_day' do
    context 'no entries' do
      it 'is zero' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/nothing.txt')
        expect(journal.average_hours_per_day).to be_zero
      end
    end

    context 'over two weeks' do
      it 'knows the average hours per 7-day-week' do
        journal = described_class.new file: Pathname.pwd.join('spec/fixtures/two_weeks.txt')
        expect(journal.average_hours_per_day).to be_instance_of BigDecimal
        expect(journal.average_hours_per_day).to be_within(0.01).of(0.14)
      end
    end
  end

  describe '#first_monday' do
    it 'is the first monday' do
      journal = described_class.new file: Pathname.pwd.join('spec/fixtures/two_weeks.txt')
      expect(journal.first_monday).to be_monday
      expect(journal.first_monday).to eq Date.new(2016, 10, 3)
    end
  end

  describe '#last_sunday' do
    it 'is the last sunday' do
      journal = described_class.new file: Pathname.pwd.join('spec/fixtures/two_weeks.txt')
      expect(journal.last_sunday).to be_sunday
      expect(journal.last_sunday).to eq Date.new(2016, 10, 16)
    end
  end

end
