# frozen_string_literal: true

RSpec.describe Deeds::Journals::Entry do

  describe '#node and #error' do
    context 'valid input' do
      it 'has the node and no error' do
        entry = described_class.new input: '2014-05-06 5m -4m'
        expect(entry.node).to be_instance_of Citrus::Match
        expect(entry.error).to be_nil
      end
    end

    context 'invalid input' do
      it 'has no node and has an error' do
        entry = described_class.new input: 'not this either'
        expect(entry.node).to be_nil
        expect(entry.error).to be_instance_of Citrus::ParseError
      end
    end

    context 'invalid date' do
      it 'keeps the node but has an error' do
        entry = described_class.new input: '2016-02-31 not this either'
        expect(entry.node).to be_instance_of Citrus::Match
        expect(entry.error).to be_instance_of Citrus::ParseError
        expect(entry.error.message).to include 'not a valid date'
      end
    end

    context 'invalid time' do
      it 'keeps the node but has an error' do
        entry = described_class.new input: '2016-02-17 7:20-25:80 5m'
        expect(entry.node).to be_instance_of Citrus::Match
        expect(entry.error).to be_instance_of Citrus::ParseError
        expect(entry.error.message).to include 'not a valid time'
      end
    end
  end

  describe '#date' do
    context 'valid date' do
      it 'is a Date instance' do
        entry = described_class.new input: '16-4-19'
        expect(entry.date).to be_instance_of Date
      end
    end

    context 'invalid date' do
      it 'is nil' do
        entry = described_class.new input: '1816-04-33'
        expect(entry.date).to be_nil
      end
    end
  end

  describe '#minutes' do
    context 'invalid input' do
      it 'is nil' do
        entry = described_class.new input: 'kaboom'
        expect(entry.minutes).to be_nil
      end
    end

    context 'no times' do
      it 'is zero' do
        entry = described_class.new input: '16-4-19'
        expect(entry.minutes).to eq 0
      end
    end

    context 'interval' do
      it 'is the amount of minutes' do
        entry = described_class.new input: '16-4-19 7:10-7:30'
        expect(entry.minutes).to eq 20
      end
    end

    context 'interval and durations' do
      it 'is the amount of minutes' do
        entry = described_class.new input: '16-4-19 7:10-7:30 -5m'
        expect(entry.minutes).to eq 15
      end
    end
  end

  describe '#activity' do
    context 'invalid input' do
      it 'is nil' do
        entry = described_class.new input: 'wham!'
        expect(entry.activity).to be_nil
      end
    end

    context 'no activity' do
      it 'is nil' do
        entry = described_class.new input: '16-4-19'
        expect(entry.activity).to be_nil
      end
    end
  end

end
